import 'dart:convert';

// ignore: import_of_legacy_library_into_null_safe
import 'package:des_plugin/des_plugin.dart' as des;
import 'package:flutter_cache/flutter_cache.dart' as cache;
import 'package:http/http.dart' as http;

import 'package:jiosaavn_wrapper/constants/strings.dart';
import 'package:jiosaavn_wrapper/modals/index.dart';

import 'package:jiosaavn_wrapper/utilities/extensions.dart';

class JioSaavnWrapper {
  ///
  ///[Cached] data of top songs.
  ///
  ///Data will be [cached] for 24 hours from fetched time
  ///
  ///cached time can be changed py passing in the [expiredAt] parameter
  ///
  ///
  static Future<TopSongs?> fetchTopSongs({int? expiredAt = 86400}) async {
    try {
      var map = await cache.remember(
        'topSongs',
        () async {
          var response = await http.get(Uri.parse(TOPSONGSURL), headers: {
            "Access-Control-Allow-Origin": "*",
          });
          return Map<String, dynamic>.from(jsonDecode(response.body));
        },
        expiredAt,
      );
      return TopSongs.fromJson(map);
    } catch (e) {
      throw e;
    }
  }

  ///
  ///Fetch [PlaylisInfo] for the supplied [playlistId]
  ///
  static Future<PlaylistInfo> fetchPlylistInfo(
      {required String playlistId}) async {
    try {
      var res = await cache.remember(
        'playlist$playlistId',
        () async {
          var response = await http.get(
            Uri.parse(playlisturl(playlistId)),
            headers: {"Accept": "application/json"},
          );
          var resEdited = (response.body).split("-->");
          return json.decode(resEdited[1]);
        },
      );
      return PlaylistInfo.fromMap(Map<String, dynamic>.from(res));
    } catch (e) {
      throw e;
    }
  }

  ///
  ///Fetch [SearchResults] for the supplied [SearchQuery]
  ///
  static Future<SearchResults> fetchSearchResults(
      {required String searchQuery}) async {
    try {
      var map = await cache.remember('searchResultsof$searchQuery', () async {
        var response = await http.get(Uri.parse(searchUrl(searchQuery)),
            headers: {"Accept": "application/json"});
        var resEdited = (response.body).split("-->");
        return json.decode(resEdited[1]);
      });
      return SearchResults.fromMap(Map<String, dynamic>.from(map));
    } catch (e) {
      throw e;
    }
  }

  ///
  ///Fetch [SongDetails] for the supplied songId
  ///

  static Future<SongDetails> fetchSongDetails({required String songId}) async {
    try {
      String songUrl = SONGDETAILURL + songId;

      var map = await cache.remember(
          'songDetails_$songId',
          () async => Map<String, dynamic>.from(jsonDecode(await http
                  .get(Uri.parse(songUrl), headers: {
                "Accept": "application/json"
              }).then((value) => value.body.split("-->")[1]))[songId]));

      var songDetails = SongDetails.fromJson(map);

      var decryptedMediaUrl =
          await decryptUrl(url: songDetails.moreInfo!.encryptedMediaUrl!);

      return songDetails.copyWith(
        moreInfo: songDetails.moreInfo!.copyWith(
          encryptedMediaUrl: decryptedMediaUrl,
        ),
        lyrics: await fetchLyrics(
          songId: songId,
          hasLyrics: songDetails.moreInfo!.hasLyrics == "true",
          hasArtistName: songDetails.moreInfo!.artistMap!.artists!.isNotEmpty,
          artist: songDetails.moreInfo!.artistMap!.artists!.first.name!,
          title: songDetails.title!,
        ),
        image: songDetails.image!.securedUrl,
      );
    } catch (e) {
      throw e;
    }
  }

  ///
  ///[Decrypt] url
  ///
  static Future<String> decryptUrl({required String url}) async {
    return (await cache.remember(
      url,
      () async {
        return await des.DesPlugin.decrypt(
          '38346591',
          url,
        );
      },
      86400,
    ))
        .toString()
        .sanitize;
  }

  ///
  /// [Fetch] Lyrics
  ///
  static Future<String?> fetchLyrics(
      {required String songId,
      required bool hasLyrics,
      required bool hasArtistName,
      required String artist,
      required String title}) async {
    try {
      return await cache.remember(
          'lyrics$songId',
          () async => hasLyrics
              ? jsonDecode(await http.get(
                  Uri.parse(LYRICSURL(songId)),
                  headers: {"Accept": "application/json"},
                ).then((value) => value.body.split('-->')[1]))['lyrics']
                  .toString()
                  .replaceAll('<br>', '\n')
              : hasArtistName
                  ? jsonDecode(await http.get(
                      Uri.parse(alernateLyricsUrl(artist: artist, name: title)),
                      headers: {"Accept": "application/json"},
                    ).then((value) => value.body))['lyrics']
                  : '');
    } catch (e) {
      throw e;
    }
  }
}
